require 'sketchup.rb'

module StableSketch
  class CrashGuard
    CONFIG_FILE ||= File.join(File.dirname(__FILE__), "lj.conf")
    @@instance = nil

    def self.instance
      @@instance ||= new
    end

    def initialize
      @backup_path = self.class.backup_directory
      Dir.mkdir(@backup_path) unless Dir.exist?(@backup_path)
      StableSketch.log_info("CrashGuard 初始化，备份路径：#{@backup_path}", @backup_path)
      @last_save = Time.now
      @monitoring_started = false
    end

    def backup_dir
      @backup_path
    end

    def start_monitoring
      @monitoring_started = true
      StableSketch.log_info("监控已启动", @backup_path)
      UI.messagebox("自动备份监控已启动！")
    end

    def optimize_graphics
      Sketchup.active_model.options["RenderSettings"]["UseHardwareAcceleration"] = true
      StableSketch.log_info("图形优化已执行", @backup_path)
      UI.messagebox("图形优化完成！")
    end

    def check_model
      StableSketch.log_info("开始清理模型", @backup_path)
      Sketchup.active_model.entities.grep(Sketchup::Edge).each { |e| e.hidden = true if e.faces.empty? }
      UI.messagebox("模型清理完成！")
    end

    def open_backup
      StableSketch.log_info("打开备份文件夹：#{@backup_path}", @backup_path)
      system("explorer \"#{@backup_path}\"") if Sketchup.platform == :platform_win
      UI.messagebox("已打开备份文件夹！")
    end

    def auto_save
      model = Sketchup.active_model
      file_path = File.join(@backup_path, "AutoSave_#{Time.now.strftime('%Y%m%d_%H%M%S')}.skp")
      model.save(file_path)
      @last_save = Time.now
      StableSketch.log_info("自动保存至：#{file_path}", @backup_path)
    end

    def self.backup_directory
      default_path = self.default_backup_directory

      if File.exist?(CONFIG_FILE)
        user_path = File.read(CONFIG_FILE, encoding: 'UTF-8').strip
        StableSketch.log_info("读取用户设置路径：#{user_path.inspect}", default_path)

        if !user_path.empty? && Dir.exist?(user_path) && File.directory?(user_path)
          StableSketch.log_info("使用用户设置的备份路径：#{user_path}", user_path)
          return user_path
        else
          StableSketch.log_warn("用户设置的备份路径无效：#{user_path}", default_path)
          if user_path =~ /module\s+StableSketch|class\s+CrashGuard|def\s+\w+/
            begin
              File.delete(CONFIG_FILE)
              StableSketch.log_info("检测到配置文件包含代码，已删除 #{CONFIG_FILE}", default_path)
              UI.messagebox("备份路径配置文件无效（包含代码），已删除。请重新设置备份路径！")
            rescue => e
              StableSketch.log_error("删除无效配置文件失败", e, default_path)
            end
          end
        end
      end

      export_path = Sketchup.read_default("Preferences", "ExportPath")
      StableSketch.log_info("读取 SketchUp 导出路径原始值：#{export_path.inspect}", default_path)
      if export_path && !export_path.empty?
        if Dir.exist?(export_path)
          StableSketch.log_info("使用 SketchUp 导出路径：#{export_path}", export_path)
          return export_path
        else
          begin
            Dir.mkdir(export_path)
            StableSketch.log_info("成功创建 SketchUp 导出路径：#{export_path}", export_path)
            return export_path
          rescue => e
            StableSketch.log_warn("无法创建导出路径：#{export_path}，错误：#{e.message}", export_path)
          end
        end
      end

      StableSketch.log_info("使用默认备份路径：#{default_path}", default_path)
      default_path
    end

    def self.default_backup_directory
      if Sketchup.platform == :platform_win
        File.join(ENV['USERPROFILE'], 'Documents', 'StableSketch_Backup')
      else
        File.expand_path('~/StableSketch_Backup')
      end
    end

    def self.set_backup_path
      current_path = File.exist?(CONFIG_FILE) ? File.read(CONFIG_FILE, encoding: 'UTF-8').strip : self.backup_directory
      prompts = ["备份路径（输入完整文件夹路径，例如 C:\\Backup）："]
      defaults = [current_path]
      result = UI.inputbox(prompts, defaults, "设置备份路径")
      return unless result

      new_path = result[0].strip
      StableSketch.log_info("用户输入备份路径：#{new_path.inspect}", self.backup_directory)

      if new_path.empty?
        StableSketch.log_warn("用户输入的路径为空，未保存", self.backup_directory)
        UI.messagebox("路径不能为空！")
        return
      end

      if new_path =~ /module\s+StableSketch|class\s+CrashGuard|def\s+\w+/
        StableSketch.log_warn("用户输入疑似代码而非路径：#{new_path}", self.backup_directory)
        UI.messagebox("操，你输入的是代码不是路径！请输入文件夹路径，例如 C:\\Backup")
        return
      end

      unless new_path =~ /^[A-Za-z]:[\/\\]|^[\/\\]|^~[\/\\]/
        StableSketch.log_warn("用户输入的路径格式无效：#{new_path}", self.backup_directory)
        UI.messagebox("操，路径格式不对！请输入类似 C:\\Backup 的文件夹路径")
        return
      end

      begin
        Dir.mkdir(new_path) unless Dir.exist?(new_path)
        if Dir.exist?(new_path) && File.directory?(new_path)
          File.write(CONFIG_FILE, new_path, encoding: 'UTF-8')
          StableSketch.log_info("备份路径已保存到 #{new_path}", new_path)  # 修改：只显示目标路径
          UI.messagebox("备份路径已设置为：#{new_path}")
          CrashGuard.instance.instance_variable_set(:@backup_path, new_path)
        else
          raise "路径无效或无法创建"
        end
      rescue => e
        StableSketch.log_error("保存备份路径失败", e, self.backup_directory)
        UI.messagebox("操，设置路径失败：#{e.message}")
      end
    end
  end
end
