# encoding: UTF-8
module StableSketch
  class FaceCounter
    def initialize
      @dialog = nil
      @active = true
      setup_observers
      create_dialog
      update_display
    end

    def count_selected_entities
      total = 0
      model = Sketchup.active_model
      processed_definitions = Set.new  # 用于组件去重

      model.selection.each do |entity|
        next unless entity.is_a?(Sketchup::ComponentInstance) || entity.is_a?(Sketchup::Group)

        if entity.is_a?(Sketchup::ComponentInstance)
          # 组件只统计一次
          definition = entity.definition
          unless processed_definitions.include?(definition)
            processed_definitions.add(definition)
            total += count_faces_recursive(definition.entities, processed_definitions)
          end
        elsif entity.is_a?(Sketchup::Group)
          # 群组每次都统计，不去重
          total += count_faces_recursive(entity.definition.entities, processed_definitions)
        end
      end
      total
    end

    def count_total_faces
      model = Sketchup.active_model
      processed_definitions = Set.new  # 用于组件去重
      count_faces_recursive(model.entities, processed_definitions)
    end

    # 递归统计面数（组件去重，群组不���重）
    def count_faces_recursive(entities, processed_definitions)
      face_count = 0

      entities.each do |entity|
        if entity.is_a?(Sketchup::Face) && entity.visible?
          face_count += 1
        elsif entity.is_a?(Sketchup::ComponentInstance)
          definition = entity.definition
          unless processed_definitions.include?(definition)
            processed_definitions.add(definition)
            face_count += count_faces_recursive(definition.entities, processed_definitions)
          end
        elsif entity.is_a?(Sketchup::Group)
          # 群组不去重，每次都统计
          face_count += count_faces_recursive(entity.definition.entities, processed_definitions)
        end
      end

      face_count
    end

    def setup_observers
      counter_instance = self

      @selection_observer = Sketchup.active_model.selection.add_observer(
        Object.new.tap do |obs|
          obs.define_singleton_method(:onSelectionBulkChange) { |*| counter_instance.update_display }
        end
      )

      @model_observer = Sketchup.active_model.add_observer(
        Object.new.tap do |obs|
          %i[onTransactionUndo onTransactionRedo onChangeEntity].each do |m|
            obs.define_singleton_method(m) { |*| counter_instance.update_display }
          end
        end
      )
    end

    def create_dialog
      return if @dialog
      
      @dialog = UI::WebDialog.new("面数统计", true, "FaceCounter", 320, 220)
      @dialog.add_action_callback("close") { stop_monitoring }
      
      html_path = File.join(File.dirname(__FILE__), "face_counter.html")
      @template = File.read(html_path, encoding: 'UTF-8')
      
      @dialog.set_html(update_html(0, 0))
      @dialog.show
    end

    def update_display
      return unless @active
      
      selected = count_selected_entities
      total = count_total_faces
      
      if @dialog.visible?
        @dialog.execute_script("updateCounts(#{selected}, #{total})")
      end
      
      Sketchup.status_text = format_status(selected, total)
    end

    def update_html(selected, total)
      @template.dup
        .gsub('{{selected}}', selected.to_s)
        .gsub('{{total}}', total.to_s)
    end

    def format_status(selected, total)
      types = Sketchup.active_model.selection.map(&:class).uniq
      icon = case types
            when [Sketchup::Group] then '📦'
            when [Sketchup::ComponentInstance] then '🧩'
            when [] then '🛑'
            else '📦+🧩'
            end
      selected > 0 ? "#{icon} 已选: #{selected}面 | 全部: #{total}面" : "#{icon} 未选择对象"
    end

    def stop_monitoring
      @active = false
      @dialog.close if @dialog
      Sketchup.active_model.selection.remove_observer(@selection_observer) rescue nil
      Sketchup.active_model.remove_observer(@model_observer) rescue nil
    end
  end
end