# StableSketch/mxrh.rb - 模型柔化（固定 40°，优化性能）

require 'sketchup.rb'

module StableSketch
  class ModelSoftener
    THRESHOLD_DEG = 40.0  # 固定最大柔化角度（度）

    def initialize
      @backup_path = CrashGuard.default_backup_directory
      soften_edges
    end

    def count_selected_entities
      total = 0
      model = Sketchup.active_model
      processed_definitions = Set.new  # 用于组件去重

      model.selection.each do |entity|
        next unless entity.is_a?(Sketchup::ComponentInstance) || entity.is_a?(Sketchup::Group)

        if entity.is_a?(Sketchup::ComponentInstance)
          # 组件只统计一次
          definition = entity.definition
          unless processed_definitions.include?(definition)
            processed_definitions.add(definition)
            total += count_faces_recursive(definition.entities, processed_definitions)
          end
        elsif entity.is_a?(Sketchup::Group)
          # 群组每次都统计，不去重
          total += count_faces_recursive(entity.definition.entities, processed_definitions)
        end
      end
      total
    end

    def count_faces_recursive(entities, processed_definitions)
      face_count = 0

      entities.each do |entity|
        if entity.is_a?(Sketchup::Face) && entity.visible?
          face_count += 1
        elsif entity.is_a?(Sketchup::ComponentInstance)
          definition = entity.definition
          unless processed_definitions.include?(definition)
            processed_definitions.add(definition)
            face_count += count_faces_recursive(definition.entities, processed_definitions)
          end
        elsif entity.is_a?(Sketchup::Group)
          # 群组不去重，每次都统计
          face_count += count_faces_recursive(entity.definition.entities, processed_definitions)
        end
      end

      face_count
    end

    def soften_edges
      model = Sketchup.active_model
      selection = model.selection

      if selection.empty?
        StableSketch.log_info("未选择任何模型，无法柔化", @backup_path)
        UI.messagebox("操，先选中模型再柔化啊！")
        return
      end

      face_count = count_selected_entities
      if face_count > 300000
        result = UI.messagebox("这个模型已超出30万面，柔化会卡很久，需要先减面！是否强制柔化，请先保存一下建模，可能需要等一会。", MB_YESNO)
        if result == IDNO
          return
        end
      end

      threshold_rad = THRESHOLD_DEG.degrees
      StableSketch.log_info("开始柔化选中的模型，角度阈值: #{THRESHOLD_DEG}°", @backup_path)
      model.start_operation("柔化边线", true)

      selection.each do |entity|
        soften_entity(entity, threshold_rad)
      end

      model.commit_operation
      model.active_view.refresh
      StableSketch.log_info("模型柔化完成", @backup_path)
    rescue => e
      model.abort_operation if model
      StableSketch.log_error("模型柔化失败: #{e.message}", e, @backup_path)
      UI.messagebox("操，柔化失败了！错误: #{e.message}")
    end

    private

    def soften_entity(entity, threshold_rad)
      edges = collect_edges(entity)
      edges.each_slice(500) do |batch|  # 分批处理
        batch.each do |edge|
          soften_edge(edge, threshold_rad)
          soften_coplanar_edge(edge)
        end
      end
    end

    def collect_edges(entity)
      edges = []
      if entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance)
        return edges if entity.locked?
        entity.definition.entities.each do |sub_entity|
          if sub_entity.is_a?(Sketchup::Edge)
            edges << sub_entity
          elsif sub_entity.is_a?(Sketchup::Group) || sub_entity.is_a?(Sketchup::ComponentInstance)
            edges.concat(collect_edges(sub_entity))
          end
        end
      elsif entity.is_a?(Sketchup::Edge)
        edges << entity
      end
      edges
    end

    def soften_edge(edge, threshold_rad)
      return unless edge.valid?
      faces = edge.faces
      return if faces.size != 2

      normal1 = faces[0].normal
      normal2 = faces[1].normal
      angle = normal1.angle_between(normal2)

      begin
        if angle <= threshold_rad
          edge.soft = true
          edge.smooth = true
        else
          edge.soft = false
        end
      rescue => e
        StableSketch.log_info("边线 #{edge.entityID} 柔化失败: #{e.message}", @backup_path)
      end
    end

    def soften_coplanar_edge(edge)
      return unless edge.valid?
      faces = edge.faces
      return if faces.size != 2

      normal1 = faces[0].normal
      normal2 = faces[1].normal
      angle = normal1.angle_between(normal2)

      begin
        if angle < 0.001
          edge.soft = true
          edge.smooth = true
        end
      rescue => e
        StableSketch.log_info("边线 #{edge.entityID} 共面柔化失败: #{e.message}", @backup_path)
      end
    end
  end
end
