require 'thread'

module StableSketch
  class CrashGuard
    def check_model
      StableSketch.log_info("check_model 开始执行", @backup_path)
      model = Sketchup.active_model
      unless model
        StableSketch.log_warn("模型清理跳过：没有活动模型", @backup_path)
        return
      end

      begin
        model.start_operation("模型体检", true)

        # 多线程任务队列
        queue = Queue.new
        threads = []

        # 统计初始信息（线程 1）
        threads << Thread.new do
          StableSketch.log_info("线程 1：统计初始信息", @backup_path)
          initial_stats = count_entities_recursive(model.entities)
          queue.push({ type: :stats, data: initial_stats, definitions: model.definitions.length,
                       materials: model.materials.length, layers: model.layers.length, styles: model.styles.count })
        end

        # 清理孤立边（线程 2）
        threads << Thread.new do
          StableSketch.log_info("线程 2：清理孤立边", @backup_path)
          deleted_edges = purge_orphaned_edges(model.entities, model)
          queue.push({ type: :edges, data: deleted_edges })
        end

        # 等待线程完成并收集结果
        initial_stats = deleted_edges = nil
        threads.each { |t| t.join }
        until queue.empty?
          result = queue.pop
          case result[:type]
          when :stats
            initial_stats = result[:data]
            initial_definitions = result[:definitions]
            initial_materials = result[:materials]
            initial_layers = result[:layers]
            initial_styles = result[:styles]
          when :edges
            deleted_edges = result[:data]
          end
        end

        StableSketch.log_info("清理了 #{deleted_edges} 条孤立边", @backup_path)

        # 清理未使用资源
        deleted_definitions = purge_unused_definitions(model)
        StableSketch.log_info("清理了 #{deleted_definitions} 个未使用的组件定义", @backup_path)

        model.materials.purge_unused
        deleted_materials = initial_materials - model.materials.length
        StableSketch.log_info("清理了 #{deleted_materials} 个未使用的材质", @backup_path)

        model.layers.purge_unused
        deleted_layers = initial_layers - model.layers.length
        StableSketch.log_info("清理了 #{deleted_layers} 个未使用的图层", @backup_path)

        model.styles.purge_unused
        deleted_styles = initial_styles - model.styles.count
        StableSketch.log_info("清理了 #{deleted_styles} 个未使用的样式", @backup_path)

        # 统计最终信息
        StableSketch.log_info("统计最终信息", @backup_path)
        final_stats = count_entities_recursive(model.entities)
        final_edges = final_stats[:edges]
        final_faces = final_stats[:faces]

        # 固定面数警告阈值：100万面
        if final_faces > 1_000_000
          StableSketch.log_info("面数警告：#{final_faces} 超过100万面阈值", @backup_path)
          UI.messagebox("警告：模型面数过多（#{final_faces} > 1,000,000），建议优化！")
        end

        # 生成统计信息
        stats_message = "模型清理完成！\n" \
                        "清理了 #{deleted_edges} 条孤立边\n" \
                        "清理了 #{deleted_definitions} 个未使用的组件定义\n" \
                        "清理了 #{deleted_materials} 个未使用的材质\n" \
                        "清理了 #{deleted_layers} 个未使用的图层\n" \
                        "清理了 #{deleted_styles} 个未使用的样式\n" \
                        "当前模型：#{final_edges} 条线，#{final_faces} 个面"
        StableSketch.log_info("准备显示统计信息：#{stats_message}", @backup_path)
        puts stats_message
        $stdout.flush
        begin
          UI.messagebox(stats_message)
        rescue => e
          StableSketch.log_error("UI.messagebox 失败", e, @backup_path)
        end

        # 无渲染优化，直接刷新视图
        model.active_view.invalidate
        StableSketch.log_info("模型清理完成，视图已更新", @backup_path)
      rescue => e
        StableSketch.log_error("模型清理失败", e, @backup_path)
      ensure
        model.commit_operation
      end
    end

    private

    # 递归统计实体（组件去重，群组不去重）
    def count_entities_recursive(entities, processed_definitions = Set.new)
      edges = 0
      faces = 0

      entities.each do |entity|
        if entity.is_a?(Sketchup::Edge) && entity.valid?
          edges += 1
        elsif entity.is_a?(Sketchup::Face) && entity.valid?
          faces += 1
        elsif entity.is_a?(Sketchup::ComponentInstance)
          # 对组件实例只统计一次其定义
          definition = entity.definition
          unless processed_definitions.include?(definition)
            processed_definitions.add(definition)
            sub_stats = count_entities_recursive(definition.entities, processed_definitions)
            edges += sub_stats[:edges]
            faces += sub_stats[:faces]
          end
        elsif entity.is_a?(Sketchup::Group)
          # 对群组每次都统计，不去重
          sub_stats = count_entities_recursive(entity.definition.entities, processed_definitions)
          edges += sub_stats[:edges]
          faces += sub_stats[:faces]
        end
      end

      { edges: edges, faces: faces }
    end

    # 清理孤立边（智能阈值）
    def purge_orphaned_edges(entities, model)
      deleted_edges = 0
      edge_lengths = []

      # 递归收集所有边长
      collect_edge_lengths(entities, edge_lengths)
      avg_length = edge_lengths.empty? ? 0 : (edge_lengths.sum / edge_lengths.size)
      threshold = avg_length * 0.1

      # 递归清理孤立边
      deleted_edges = erase_orphaned_edges(entities, threshold)

      StableSketch.log_info("孤立边清理阈值：#{threshold.round(2)}，平均边长：#{avg_length.round(2)}", @backup_path)
      deleted_edges
    end

    # 辅助方法：递归收集边长（组件去重，群组不去重）
    def collect_edge_lengths(entities, edge_lengths, processed_definitions = Set.new)
      entities.each do |entity|
        if entity.is_a?(Sketchup::Edge) && entity.valid?
          edge_lengths << entity.length
        elsif entity.is_a?(Sketchup::ComponentInstance)
          definition = entity.definition
          unless processed_definitions.include?(definition)
            processed_definitions.add(definition)
            collect_edge_lengths(definition.entities, edge_lengths, processed_definitions)
          end
        elsif entity.is_a?(Sketchup::Group)
          # 群组不跳过，每次都收集
          collect_edge_lengths(entity.definition.entities, edge_lengths, processed_definitions)
        end
      end
    end

    # 辅助方法：递归清理孤立边（组件去重，群组不去重）
    def erase_orphaned_edges(entities, threshold, processed_definitions = Set.new)
      deleted_edges = 0
      entities_to_erase = []

      entities.each do |entity|
        next unless entity.valid?  # 添加有效性检查
        
        if entity.is_a?(Sketchup::Edge) && entity.faces.empty? && entity.length < threshold
          entities_to_erase << entity
          deleted_edges += 1
        elsif entity.is_a?(Sketchup::ComponentInstance)
          definition = entity.definition
          if definition && !processed_definitions.include?(definition)
            processed_definitions.add(definition)
            deleted_edges += erase_orphaned_edges(definition.entities, threshold, processed_definitions)
          end
        elsif entity.is_a?(Sketchup::Group)
          # 群组不跳过，每次都清理
          deleted_edges += erase_orphaned_edges(entity.definition.entities, threshold, processed_definitions)
        end
      end

      # 批量删除孤立边，添加有效性检查
      entities_to_erase.each do |edge|
        if edge.valid?
          begin
            edge.erase!
          rescue => e
            StableSketch.log_error("删除孤立边失败", e, @backup_path)
          end
        end
      end
      
      deleted_edges
    end

    def purge_unused_definitions(model)
      initial_count = model.definitions.length
      model.definitions.purge_unused
      initial_count - model.definitions.length
    end
  end
end