require 'net/http'
require 'uri'

module StableSketch
  class HotUpdateChecker
    REMOTE_PAGE_URL = "http://www.cdsjcc.cn/stablesketch"  # 使用正确的版本页面
    DOWNLOAD_PAGE = "http://www.cdsjcc.cn/stablesketch"

    def initialize
      @backup_path = CrashGuard.backup_directory
      @local_version = StableSketch::VERSION  # "5.3.2025"
      StableSketch.log_info("HotUpdateChecker 初始化，备份路径：#{@backup_path}", @backup_path)
    end

    def check_update
      StableSketch.log_info("检查热更新开始", @backup_path)
      
      begin
        # 获取远程版本号
        remote_version = fetch_remote_version
        StableSketch.log_info("本地版本: #{@local_version}, 远程版本: #{remote_version}", @backup_path)

        # 比较版本号
        if remote_version && version_greater?(remote_version, @local_version)
          StableSketch.log_info("检测到新版本: #{remote_version}", @backup_path)
          prompt_update(remote_version)
        else
          StableSketch.log_info("已是最新版本: #{@local_version}", @backup_path)
          UI.messagebox("操，你已经是最新版本 #{@local_version} 了！")  # 仅确认，无跳转询问
        end
      rescue => e
        StableSketch.log_error("检查更新失败: #{e.message}", e, @backup_path)
        prompt_failure("操，检查更新失败了！错误: #{e.message}\n是否打开下载页面查看最新版本？")
      end
    end

    private

    def fetch_remote_version
      uri = URI(REMOTE_PAGE_URL)
      http = Net::HTTP.new(uri.host, uri.port)
      request = Net::HTTP::Get.new(uri)
      request['User-Agent'] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"

      # 跟随重定向
      max_redirects = 5
      redirects = 0

      while redirects < max_redirects
        response = http.request(request)
        
        if response.is_a?(Net::HTTPSuccess)
          html = response.body
          match = html.match(/StableSketch_(\d+\.\d+\.\d+)/)
          if match && valid_version?(match[1])
            return match[1]  # 返回 "5.4.2025"
          else
            raise "无法从页面提取有效版本号"
          end
        elsif response.is_a?(Net::HTTPRedirection)
          new_location = response['Location']
          StableSketch.log_info("重定向到: #{new_location}", @backup_path)
          uri = URI.parse(new_location)
          http = Net::HTTP.new(uri.host, uri.port)
          request = Net::HTTP::Get.new(uri)
          request['User-Agent'] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
          redirects += 1
        else
          raise "无法获取远程版本: #{response.code} #{response.message}"
        end
      end
      
      raise "重定向次数过多，可能是循环重定向"
    end

    def valid_version?(version)
      version =~ /^\d+\.\d+\.\d+$/
    end

    def version_greater?(remote, local)
      remote_parts = remote.split('.').map(&:to_i)
      local_parts = local.split('.').map(&:to_i)
      return true if remote_parts[2] > local_parts[2]
      return false if remote_parts[2] < local_parts[2]
      return true if remote_parts[0] > local_parts[0]
      return false if remote_parts[0] < local_parts[0]
      remote_parts[1] > local_parts[1]
    end

    def prompt_update(new_version)
      message = "操，检测到新版本 #{new_version}！\n当前版本 #{@local_version} 已过时，赶紧去更新吧！"
      result = UI.messagebox(message + "\n\n是否跳转到下载页面？", MB_YESNO)
      if result == IDYES
        StableSketch.log_info("用户选择跳转到下载页面", @backup_path)
        UI.openURL(DOWNLOAD_PAGE)
      else
        StableSketch.log_info("用户取消更新", @backup_path)
      end
    end

    def prompt_failure(message)
      result = UI.messagebox(message, MB_YESNO)
      if result == IDYES
        StableSketch.log_info("用户选择跳转到下载页面查看版本", @backup_path)
        UI.openURL(DOWNLOAD_PAGE)
      else
        StableSketch.log_info("用户取消跳转", @backup_path)
      end
    end
  end
end